@@ -22,6 +22,7 @@
 #ifndef DC_IOSTREAM_H
 #define DC_IOSTREAM_H
 
+#include <stddef.h>
 #include <libdivecomputer/common.h>
 #include <libdivecomputer/context.h>
 
@@ -82,6 +83,15 @@ typedef enum dc_line_t {
 	DC_LINE_RNG = 0x08, /**< Ring indicator */
 } dc_line_t;
 
+/**
+ * Get the transport type.
+ *
+ * @param[in]  iostream  A valid I/O stream.
+ * @returns The transport type of the I/O stream.
+ */
+dc_transport_t
+dc_iostream_get_transport (dc_iostream_t *iostream);
+
 /**
  * Set the read timeout.
  *
@@ -113,22 +123,6 @@ typedef enum dc_line_t {
 dc_status_t
 dc_iostream_set_timeout (dc_iostream_t *iostream, int timeout);
 
-/**
- * Set the receive latency.
- *
- * The effect of this setting is highly platform and driver specific. On
- * Windows it does nothing at all, on Linux it controls the low latency
- * flag (e.g. only zero vs non-zero latency), and on Mac OS X it sets
- * the receive latency as requested.
- *
- * @param[in]  iostream  A valid I/O stream.
- * @param[in]  value     The latency in milliseconds.
- * @returns #DC_STATUS_SUCCESS on success, or another #dc_status_t code
- * on failure.
- */
-dc_status_t
-dc_iostream_set_latency (dc_iostream_t *iostream, unsigned int value);
-
 /**
  * Set the state of the break condition.
  *
@@ -201,6 +195,36 @@ dc_iostream_get_available (dc_iostream_t *iostream, size_t *value);
 dc_status_t
 dc_iostream_configure (dc_iostream_t *iostream, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol);
 
+/**
+ * Poll the I/O stream for available data.
+ *
+ * There are three distinct modes available:
+ *
+ *  1. Blocking (timeout < 0):
+ *
+ *     The poll operation is blocked until one or more bytes have been
+ *     received. If no bytes are received, the operation will block
+ *     forever.
+ *
+ *  2. Non-blocking (timeout == 0):
+ *
+ *     The poll operation returns immediately, even if no bytes have
+ *     been received.
+ *
+ *  3. Timeout (timeout > 0):
+ *
+ *     The poll operation is blocked until one or more bytes have been
+ *     received. If no bytes are received within the specified amount of
+ *     time, the operation will return with a timeout.
+ *
+ * @param[in]  iostream  A valid I/O stream.
+ * @param[in]  timeout   The timeout in milliseconds.
+ * @returns #DC_STATUS_SUCCESS on success, #DC_STATUS_TIMEOUT on
+ * timeout, or another #dc_status_t code on failure.
+ */
+dc_status_t
+dc_iostream_poll (dc_iostream_t *iostream, int timeout);
+
 /**
  * Read data from the I/O stream.
  *
@@ -229,6 +253,19 @@ dc_iostream_read (dc_iostream_t *iostream, void *data, size_t size, size_t *actu
 dc_status_t
 dc_iostream_write (dc_iostream_t *iostream, const void *data, size_t size, size_t *actual);
 
+/**
+ * Perform an I/O stream specific request.
+ *
+ * @param[in]  iostream  A valid I/O stream.
+ * @param[in]  request   The request to perform.
+ * @param[in,out]  data  The request specific data.
+ * @param[in]  size      The size of the request specific data.
+ * @returns #DC_STATUS_SUCCESS on success, or another #dc_status_t code
+ * on failure.
+ */
+dc_status_t
+dc_iostream_ioctl (dc_iostream_t *iostream, unsigned int request, void *data, size_t size);
+
 /**
  * Flush the internal output buffer and wait until the data has been
  * transmitted.
@@ -273,6 +310,22 @@ dc_iostream_sleep (dc_iostream_t *iostream, unsigned int milliseconds);
 dc_status_t
 dc_iostream_close (dc_iostream_t *iostream);
 
+/**
+ * Get the name of the device
+ *
+ * @param[in]  iostream  A valid I/O stream.
+ * @returns a NUL-terminated constant C string that has a lifetime
+ * until the close of the iostream, or NUL if no name exists.
+ *
+ * The name depends on the iostream transport. For a BLE device, it
+ * is the name of the device during device discovery.
+ */
+const char *
+dc_iostream_get_name (dc_iostream_t *iostream);
+
+dc_status_t
+dc_usb_storage_open (dc_iostream_t **out, dc_context_t *context, const char *name);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
